/*
 * Copyright LWJGL. All rights reserved.
 * License terms: http://lwjgl.org/license.php
 * MACHINE GENERATED FILE, DO NOT EDIT
 */
package org.lwjgl.egl;

import java.nio.*;

import org.lwjgl.*;
import org.lwjgl.system.*;

import static org.lwjgl.system.Checks.*;
import static org.lwjgl.system.JNI.*;
import static org.lwjgl.system.MemoryUtil.*;
import static org.lwjgl.system.Pointer.*;

/**
 * Native bindings to the <a href="https://www.khronos.org/registry/egl/extensions/KHR/EGL_KHR_debug.txt">KHR_debug</a> extension.
 * 
 * <p>This extension allows EGL to notify applications when various events occur that may be useful during application development and debugging.</p>
 * 
 * <p>These events are represented in the form of debug messages with a human-readable string representation. Examples of debug events include errors due to
 * incorrect use of the EGL API, warnings of undefined behavior, and performance warnings.</p>
 * 
 * <p>The "type" of the message roughly identifies the nature of the event that caused the message. Examples include input errors, performance information,
 * or warnings about undefined behavior.</p>
 * 
 * <p>Messages are communicated to the application through an application- defined callback function that is called by the EGL implementation on each debug
 * message. The motivation for the callback routine is to free application developers from actively having to query whether an EGL error, or any other
 * debuggable event has happened after each call to a EGL function. With a callback, developers can keep their code free of debug checks, set breakpoints
 * in the callback function, and only have to react to messages as they occur. The callback also offers much more information than just an error code.</p>
 * 
 * <p>To control the volume of debug output, types of messages can be enabled or disabled. The mechanism is controlled by attributes passed to EGL. The state
 * of the message type control can be queried.</p>
 * 
 * <p>Debug output can be enabled and disabled by changing the callback function. {@code NULL} will disable the feature while a valid function pointer will enable
 * it.</p>
 * 
 * <p>Finally, this extension defines a mechanism for EGL applications to label their objects (contexts, surfaces, syncs, etc.) with a pointer to an
 * application provided structure. This pointer can be a descriptive string, identifier or pointer to a structure. This enables the application to
 * associate the EGL object with application information. EGL will not interpret this pointer as a string or any other meaning - just attach to the object
 * and pass back in the callback when that object is the primary object of an event.</p>
 */
public class KHRDebug {

	/** Tokens accepted by the {@code objectType} parameter of function {@link #eglLabelObjectKHR LabelObjectKHR}. */
	public static final int
		EGL_OBJECT_THREAD_KHR  = 0x33B0,
		EGL_OBJECT_DISPLAY_KHR = 0x33B1,
		EGL_OBJECT_CONTEXT_KHR = 0x33B2,
		EGL_OBJECT_SURFACE_KHR = 0x33B3,
		EGL_OBJECT_IMAGE_KHR   = 0x33B4,
		EGL_OBJECT_SYNC_KHR    = 0x33B5,
		EGL_OBJECT_STREAM_KHR  = 0x33B6;

	/**
	 * Tokens provided by the {@code messageType} parameter of EGLDEBUGPROCKHR or the attributes input to {@link #eglDebugMessageControlKHR DebugMessageControlKHR} or attribute of
	 * {@link #eglQueryDebugKHR QueryDebugKHR}.
	 */
	public static final int
		EGL_DEBUG_MSG_CRITICAL_KHR = 0x33B9,
		EGL_DEBUG_MSG_ERROR_KHR    = 0x33BA,
		EGL_DEBUG_MSG_WARN_KHR     = 0x33BB,
		EGL_DEBUG_MSG_INFO_KHR     = 0x33BC;

	/** Tokens provided by the input attribute to eglQueryDebugKHR. */
	public static final int EGL_DEBUG_CALLBACK_KHR = 0x33B8;

	/** Function address. */
	@JavadocExclude
	public final long
		DebugMessageControlKHR,
		QueryDebugKHR,
		LabelObjectKHR;

	@JavadocExclude
	protected KHRDebug() {
		throw new UnsupportedOperationException();
	}

	@JavadocExclude
	public KHRDebug(FunctionProvider provider) {
		DebugMessageControlKHR = provider.getFunctionAddress("eglDebugMessageControlKHR");
		QueryDebugKHR = provider.getFunctionAddress("eglQueryDebugKHR");
		LabelObjectKHR = provider.getFunctionAddress("eglLabelObjectKHR");
	}

	// --- [ Function Addresses ] ---

	/** Returns the {@link KHRDebug} instance. */
	public static KHRDebug getInstance() {
		return getInstance(EGL.getCapabilities());
	}

	/** Returns the {@link KHRDebug} instance of the specified {@link EGLCapabilities}. */
	public static KHRDebug getInstance(EGLCapabilities caps) {
		return checkFunctionality(caps.__KHRDebug);
	}

	// --- [ eglDebugMessageControlKHR ] ---

	/** Unsafe version of {@link #eglDebugMessageControlKHR DebugMessageControlKHR} */
	@JavadocExclude
	public static int neglDebugMessageControlKHR(long callback, long attrib_list) {
		long __functionAddress = getInstance().DebugMessageControlKHR;
		return callPPI(__functionAddress, callback, attrib_list);
	}

	public static int eglDebugMessageControlKHR(EGLDebugMessageKHRCallback callback, ByteBuffer attrib_list) {
		return neglDebugMessageControlKHR(callback.address(), memAddressSafe(attrib_list));
	}

	/** Alternative version of: {@link #eglDebugMessageControlKHR DebugMessageControlKHR} */
	public static int eglDebugMessageControlKHR(EGLDebugMessageKHRCallback callback, PointerBuffer attrib_list) {
		return neglDebugMessageControlKHR(callback.address(), memAddressSafe(attrib_list));
	}

	// --- [ eglQueryDebugKHR ] ---

	/** Unsafe version of {@link #eglQueryDebugKHR QueryDebugKHR} */
	@JavadocExclude
	public static boolean neglQueryDebugKHR(int attribute, long value) {
		long __functionAddress = getInstance().QueryDebugKHR;
		return callIPZ(__functionAddress, attribute, value);
	}

	public static boolean eglQueryDebugKHR(int attribute, ByteBuffer value) {
		if ( CHECKS )
			checkBuffer(value, 1 << POINTER_SHIFT);
		return neglQueryDebugKHR(attribute, memAddress(value));
	}

	/** Alternative version of: {@link #eglQueryDebugKHR QueryDebugKHR} */
	public static boolean eglQueryDebugKHR(int attribute, PointerBuffer value) {
		if ( CHECKS )
			checkBuffer(value, 1);
		return neglQueryDebugKHR(attribute, memAddress(value));
	}

	// --- [ eglLabelObjectKHR ] ---

	public static int eglLabelObjectKHR(long display, int objectType, long object, long label) {
		long __functionAddress = getInstance().LabelObjectKHR;
		if ( CHECKS ) {
			checkPointer(display);
			checkPointer(object);
			checkPointer(label);
		}
		return callPIPPI(__functionAddress, display, objectType, object, label);
	}

}