/*
 * Copyright LWJGL. All rights reserved.
 * License terms: http://lwjgl.org/license.php
 * MACHINE GENERATED FILE, DO NOT EDIT
 */
package org.lwjgl.system;

/**
 * This class contains native methods that can be used to call dynamically loaded functions. It is used internally by the LWJGL bindings, but can also be used
 * to call other dynamically loaded functions. Not all possible signatures are available, only those needed by the LWJGL bindings. To call a function that does
 * not have a matching JNI method, {@link org.lwjgl.system.libffi.LibFFI LibFFI} can used.
 *
 * <p>All JNI methods in this class take an extra parameter, called {@code __functionAddress}. This must be a valid pointer to a native function with a
 * matching signature. The C language does not support method overloading, so each method has a postfix that matches the
 * <a href="http://docs.oracle.com/javase/8/docs/technotes/guides/jni/spec/types.html#type_signatures">JNI type signature</a> it supports. Only the primitive
 * type signatures are used, plus a 'P' type that corresponds to a pointer value. Pointer values are mapped to Java longs, but are properly converted in native
 * code on 32bit architectures.</p>
 *
 * <p>Methods with the {@code invoke} prefix will invoke the native function with the default calling convention. Methods with the {@code call} prefix will
 * invoke the native function with the {@code __stdcall} calling convention on Windows and the default calling convention on other systems.</p>
 */
public final class JNI {

	private JNI() {}

	public static native void callBBBBV(long __functionAddress, byte param0, byte param1, byte param2, byte param3);
	public static native void callBBBV(long __functionAddress, byte param0, byte param1, byte param2);
	public static native void callBV(long __functionAddress, byte param0);
	public static native void callDDDDDDV(long __functionAddress, double param0, double param1, double param2, double param3, double param4, double param5);
	public static native void callDDDDV(long __functionAddress, double param0, double param1, double param2, double param3);
	public static native void callDDDV(long __functionAddress, double param0, double param1, double param2);
	public static native void callDDV(long __functionAddress, double param0, double param1);
	public static native void callDV(long __functionAddress, double param0);
	public static native void callFFFFFFFFV(long __functionAddress, float param0, float param1, float param2, float param3, float param4, float param5, float param6, float param7);
	public static native void callFFFFV(long __functionAddress, float param0, float param1, float param2, float param3);
	public static native void callFFFV(long __functionAddress, float param0, float param1, float param2);
	public static native void callFFV(long __functionAddress, float param0, float param1);
	public static native void callFV(long __functionAddress, float param0);
	public static native void callFZV(long __functionAddress, float param0, boolean param1);
	public static native int callI(long __functionAddress);
	public static native void callIBBBBV(long __functionAddress, int param0, byte param1, byte param2, byte param3, byte param4);
	public static native void callIDDDDDDV(long __functionAddress, int param0, double param1, double param2, double param3, double param4, double param5, double param6);
	public static native void callIDDDDV(long __functionAddress, int param0, double param1, double param2, double param3, double param4);
	public static native void callIDDDV(long __functionAddress, int param0, double param1, double param2, double param3);
	public static native void callIDDIDDV(long __functionAddress, int param0, double param1, double param2, int param3, double param4, double param5);
	public static native void callIDDIIDDIIPV(long __functionAddress, int param0, double param1, double param2, int param3, int param4, double param5, double param6, int param7, int param8, long param9);
	public static native void callIDDIIPV(long __functionAddress, int param0, double param1, double param2, int param3, int param4, long param5);
	public static native void callIDDV(long __functionAddress, int param0, double param1, double param2);
	public static native void callIDV(long __functionAddress, int param0, double param1);
	public static native void callIFFFFV(long __functionAddress, int param0, float param1, float param2, float param3, float param4);
	public static native long callIFFFP(long __functionAddress, int param0, float param1, float param2, float param3);
	public static native void callIFFFV(long __functionAddress, int param0, float param1, float param2, float param3);
	public static native void callIFFIFFV(long __functionAddress, int param0, float param1, float param2, int param3, float param4, float param5);
	public static native void callIFFIIFFIIPV(long __functionAddress, int param0, float param1, float param2, int param3, int param4, float param5, float param6, int param7, int param8, long param9);
	public static native void callIFFIIPV(long __functionAddress, int param0, float param1, float param2, int param3, int param4, long param5);
	public static native void callIFFV(long __functionAddress, int param0, float param1, float param2);
	public static native boolean callIFFZ(long __functionAddress, int param0, float param1, float param2);
	public static native void callIFV(long __functionAddress, int param0, float param1);
	public static native int callII(long __functionAddress, int param0);
	public static native void callIIDDDDV(long __functionAddress, int param0, int param1, double param2, double param3, double param4, double param5);
	public static native void callIIDDDV(long __functionAddress, int param0, int param1, double param2, double param3, double param4);
	public static native void callIIDDV(long __functionAddress, int param0, int param1, double param2, double param3);
	public static native void callIIDV(long __functionAddress, int param0, int param1, double param2);
	public static native void callIIFFFFFFFFFV(long __functionAddress, int param0, int param1, float param2, float param3, float param4, float param5, float param6, float param7, float param8, float param9, float param10);
	public static native void callIIFFFFPV(long __functionAddress, int param0, int param1, float param2, float param3, float param4, float param5, long param6);
	public static native void callIIFFFFV(long __functionAddress, int param0, int param1, float param2, float param3, float param4, float param5);
	public static native void callIIFFFV(long __functionAddress, int param0, int param1, float param2, float param3, float param4);
	public static native void callIIFFV(long __functionAddress, int param0, int param1, float param2, float param3);
	public static native boolean callIIFFZ(long __functionAddress, int param0, int param1, float param2, float param3);
	public static native void callIIFIV(long __functionAddress, int param0, int param1, float param2, int param3);
	public static native void callIIFV(long __functionAddress, int param0, int param1, float param2);
	public static native int callIII(long __functionAddress, int param0, int param1);
	public static native void callIIIDDDDV(long __functionAddress, int param0, int param1, int param2, double param3, double param4, double param5, double param6);
	public static native void callIIIDV(long __functionAddress, int param0, int param1, int param2, double param3);
	public static native float callIIIF(long __functionAddress, int param0, int param1, int param2);
	public static native void callIIIFFFFV(long __functionAddress, int param0, int param1, int param2, float param3, float param4, float param5, float param6);
	public static native void callIIIFIV(long __functionAddress, int param0, int param1, int param2, float param3, int param4);
	public static native boolean callIIIFPPPPZ(long __functionAddress, int param0, int param1, int param2, float param3, long param4, long param5, long param6, long param7);
	public static native void callIIIFV(long __functionAddress, int param0, int param1, int param2, float param3);
	public static native void callIIIIIIIIIIIIIIIV(long __functionAddress, int param0, int param1, int param2, int param3, int param4, int param5, int param6, int param7, int param8, int param9, int param10, int param11, int param12, int param13, int param14);
	public static native void callIIIIIIIIIIIIV(long __functionAddress, int param0, int param1, int param2, int param3, int param4, int param5, int param6, int param7, int param8, int param9, int param10, int param11);
	public static native void callIIIIIIIIIIIPV(long __functionAddress, int param0, int param1, int param2, int param3, int param4, int param5, int param6, int param7, int param8, int param9, int param10, long param11);
	public static native void callIIIIIIIIIIPV(long __functionAddress, int param0, int param1, int param2, int param3, int param4, int param5, int param6, int param7, int param8, int param9, long param10);
	public static native void callIIIIIIIIIIV(long __functionAddress, int param0, int param1, int param2, int param3, int param4, int param5, int param6, int param7, int param8, int param9);
	public static native void callIIIIIIIIIPV(long __functionAddress, int param0, int param1, int param2, int param3, int param4, int param5, int param6, int param7, int param8, long param9);
	public static native void callIIIIIIIIIV(long __functionAddress, int param0, int param1, int param2, int param3, int param4, int param5, int param6, int param7, int param8);
	public static native void callIIIIIIIIPV(long __functionAddress, int param0, int param1, int param2, int param3, int param4, int param5, int param6, int param7, long param8);
	public static native void callIIIIIIIIV(long __functionAddress, int param0, int param1, int param2, int param3, int param4, int param5, int param6, int param7);
	public static native void callIIIIIIIIZV(long __functionAddress, int param0, int param1, int param2, int param3, int param4, int param5, int param6, int param7, boolean param8);
	public static native void callIIIIIIIPV(long __functionAddress, int param0, int param1, int param2, int param3, int param4, int param5, int param6, long param7);
	public static native void callIIIIIIIV(long __functionAddress, int param0, int param1, int param2, int param3, int param4, int param5, int param6);
	public static native void callIIIIIIIZV(long __functionAddress, int param0, int param1, int param2, int param3, int param4, int param5, int param6, boolean param7);
	public static native void callIIIIIIPPV(long __functionAddress, int param0, int param1, int param2, int param3, int param4, int param5, long param6, long param7);
	public static native void callIIIIIIPV(long __functionAddress, int param0, int param1, int param2, int param3, int param4, int param5, long param6);
	public static native void callIIIIIIV(long __functionAddress, int param0, int param1, int param2, int param3, int param4, int param5);
	public static native void callIIIIIIZV(long __functionAddress, int param0, int param1, int param2, int param3, int param4, int param5, boolean param6);
	public static native int callIIIIIPI(long __functionAddress, int param0, int param1, int param2, int param3, int param4, long param5);
	public static native void callIIIIIPIV(long __functionAddress, int param0, int param1, int param2, int param3, int param4, long param5, int param6);
	public static native void callIIIIIPV(long __functionAddress, int param0, int param1, int param2, int param3, int param4, long param5);
	public static native void callIIIIIV(long __functionAddress, int param0, int param1, int param2, int param3, int param4);
	public static native void callIIIIIZIPV(long __functionAddress, int param0, int param1, int param2, int param3, int param4, boolean param5, int param6, long param7);
	public static native void callIIIIIZV(long __functionAddress, int param0, int param1, int param2, int param3, int param4, boolean param5);
	public static native int callIIIIPI(long __functionAddress, int param0, int param1, int param2, int param3, long param4);
	public static native void callIIIIPIIPV(long __functionAddress, int param0, int param1, int param2, int param3, long param4, int param5, int param6, long param7);
	public static native void callIIIIPIPPV(long __functionAddress, int param0, int param1, int param2, int param3, long param4, int param5, long param6, long param7);
	public static native void callIIIIPPV(long __functionAddress, int param0, int param1, int param2, int param3, long param4, long param5);
	public static native void callIIIIPV(long __functionAddress, int param0, int param1, int param2, int param3, long param4);
	public static native void callIIIIPZV(long __functionAddress, int param0, int param1, int param2, int param3, long param4, boolean param5);
	public static native void callIIIIV(long __functionAddress, int param0, int param1, int param2, int param3);
	public static native void callIIIIZIV(long __functionAddress, int param0, int param1, int param2, int param3, boolean param4, int param5);
	public static native void callIIIPIFFIPV(long __functionAddress, int param0, int param1, int param2, long param3, int param4, float param5, float param6, int param7, long param8);
	public static native void callIIIPIIIV(long __functionAddress, int param0, int param1, int param2, long param3, int param4, int param5, int param6);
	public static native void callIIIPIIPV(long __functionAddress, int param0, int param1, int param2, long param3, int param4, int param5, long param6);
	public static native void callIIIPIIV(long __functionAddress, int param0, int param1, int param2, long param3, int param4, int param5);
	public static native void callIIIPIPPPPPPV(long __functionAddress, int param0, int param1, int param2, long param3, int param4, long param5, long param6, long param7, long param8, long param9, long param10);
	public static native void callIIIPIV(long __functionAddress, int param0, int param1, int param2, long param3, int param4);
	public static native long callIIIPPP(long __functionAddress, int param0, int param1, int param2, long param3, long param4);
	public static native void callIIIPPPPV(long __functionAddress, int param0, int param1, int param2, long param3, long param4, long param5, long param6);
	public static native void callIIIPPPV(long __functionAddress, int param0, int param1, int param2, long param3, long param4, long param5);
	public static native void callIIIPPV(long __functionAddress, int param0, int param1, int param2, long param3, long param4);
	public static native void callIIIPV(long __functionAddress, int param0, int param1, int param2, long param3);
	public static native void callIIIPZV(long __functionAddress, int param0, int param1, int param2, long param3, boolean param4);
	public static native void callIIIV(long __functionAddress, int param0, int param1, int param2);
	public static native void callIIIZIIIV(long __functionAddress, int param0, int param1, int param2, boolean param3, int param4, int param5, int param6);
	public static native void callIIIZIPV(long __functionAddress, int param0, int param1, int param2, boolean param3, int param4, long param5);
	public static native void callIIIZIV(long __functionAddress, int param0, int param1, int param2, boolean param3, int param4);
	public static native void callIIIZPV(long __functionAddress, int param0, int param1, int param2, boolean param3, long param4);
	public static native void callIIIZV(long __functionAddress, int param0, int param1, int param2, boolean param3);
	public static native long callIIJ(long __functionAddress, int param0, int param1);
	public static native void callIIJJJJV(long __functionAddress, int param0, int param1, long param2, long param3, long param4, long param5);
	public static native void callIIJJJV(long __functionAddress, int param0, int param1, long param2, long param3, long param4);
	public static native void callIIJJV(long __functionAddress, int param0, int param1, long param2, long param3);
	public static native void callIIJPV(long __functionAddress, int param0, int param1, long param2, long param3);
	public static native void callIIJV(long __functionAddress, int param0, int param1, long param2);
	public static native long callIIP(long __functionAddress, int param0, int param1);
	public static native int callIIPI(long __functionAddress, int param0, int param1, long param2);
	public static native int callIIPIIIIFI(long __functionAddress, int param0, int param1, long param2, int param3, int param4, int param5, int param6, float param7);
	public static native void callIIPIIIIIFV(long __functionAddress, int param0, int param1, long param2, int param3, int param4, int param5, int param6, int param7, float param8);
	public static native void callIIPIIIIIPV(long __functionAddress, int param0, int param1, long param2, int param3, int param4, int param5, int param6, int param7, long param8);
	public static native void callIIPIIIIPV(long __functionAddress, int param0, int param1, long param2, int param3, int param4, int param5, int param6, long param7);
	public static native void callIIPIIIIV(long __functionAddress, int param0, int param1, long param2, int param3, int param4, int param5, int param6);
	public static native void callIIPIIIPIIFV(long __functionAddress, int param0, int param1, long param2, int param3, int param4, int param5, long param6, int param7, int param8, float param9);
	public static native void callIIPIIIPV(long __functionAddress, int param0, int param1, long param2, int param3, int param4, int param5, long param6);
	public static native void callIIPIIIV(long __functionAddress, int param0, int param1, long param2, int param3, int param4, int param5);
	public static native void callIIPIIPV(long __functionAddress, int param0, int param1, long param2, int param3, int param4, long param5);
	public static native void callIIPIIV(long __functionAddress, int param0, int param1, long param2, int param3, int param4);
	public static native void callIIPIPV(long __functionAddress, int param0, int param1, long param2, int param3, long param4);
	public static native void callIIPIV(long __functionAddress, int param0, int param1, long param2, int param3);
	public static native int callIIPPIIIIFI(long __functionAddress, int param0, int param1, long param2, long param3, int param4, int param5, int param6, int param7, float param8);
	public static native void callIIPPIIIV(long __functionAddress, int param0, int param1, long param2, long param3, int param4, int param5, int param6);
	public static native void callIIPPIIPV(long __functionAddress, int param0, int param1, long param2, long param3, int param4, int param5, long param6);
	public static native void callIIPPIIV(long __functionAddress, int param0, int param1, long param2, long param3, int param4, int param5);
	public static native void callIIPPIV(long __functionAddress, int param0, int param1, long param2, long param3, int param4);
	public static native void callIIPPPPIV(long __functionAddress, int param0, int param1, long param2, long param3, long param4, long param5, int param6);
	public static native int callIIPPPPPI(long __functionAddress, int param0, int param1, long param2, long param3, long param4, long param5, long param6);
	public static native int callIIPPPPPPI(long __functionAddress, int param0, int param1, long param2, long param3, long param4, long param5, long param6, long param7);
	public static native void callIIPPPPPV(long __functionAddress, int param0, int param1, long param2, long param3, long param4, long param5, long param6);
	public static native void callIIPPPV(long __functionAddress, int param0, int param1, long param2, long param3, long param4);
	public static native void callIIPPV(long __functionAddress, int param0, int param1, long param2, long param3);
	public static native void callIIPV(long __functionAddress, int param0, int param1, long param2);
	public static native void callIIV(long __functionAddress, int param0, int param1);
	public static native boolean callIIZ(long __functionAddress, int param0, int param1);
	public static native long callIIZIIJ(long __functionAddress, int param0, int param1, boolean param2, int param3, int param4);
	public static native void callIIZIV(long __functionAddress, int param0, int param1, boolean param2, int param3);
	public static native void callIIZPV(long __functionAddress, int param0, int param1, boolean param2, long param3);
	public static native void callIIZV(long __functionAddress, int param0, int param1, boolean param2);
	public static native long callIJ(long __functionAddress, int param0);
	public static native void callIJJJJV(long __functionAddress, int param0, long param1, long param2, long param3, long param4);
	public static native void callIJJJV(long __functionAddress, int param0, long param1, long param2, long param3);
	public static native void callIJJV(long __functionAddress, int param0, long param1, long param2);
	public static native void callIJV(long __functionAddress, int param0, long param1);
	public static native long callIP(long __functionAddress, int param0);
	public static native int callIPI(long __functionAddress, int param0, long param1);
	public static native int callIPIIFII(long __functionAddress, int param0, long param1, int param2, int param3, float param4, int param5);
	public static native void callIPIIIV(long __functionAddress, int param0, long param1, int param2, int param3, int param4);
	public static native void callIPIIV(long __functionAddress, int param0, long param1, int param2, int param3);
	public static native long callIPIP(long __functionAddress, int param0, long param1, int param2);
	public static native void callIPIPIPV(long __functionAddress, int param0, long param1, int param2, long param3, int param4, long param5);
	public static native void callIPIPIV(long __functionAddress, int param0, long param1, int param2, long param3, int param4);
	public static native void callIPIPPV(long __functionAddress, int param0, long param1, int param2, long param3, long param4);
	public static native void callIPIPV(long __functionAddress, int param0, long param1, int param2, long param3);
	public static native void callIPIV(long __functionAddress, int param0, long param1, int param2);
	public static native long callIPP(long __functionAddress, int param0, long param1);
	public static native int callIPPI(long __functionAddress, int param0, long param1, long param2);
	public static native void callIPPIIIV(long __functionAddress, int param0, long param1, long param2, int param3, int param4, int param5);
	public static native void callIPPIIV(long __functionAddress, int param0, long param1, long param2, int param3, int param4);
	public static native long callIPPIP(long __functionAddress, int param0, long param1, long param2, int param3);
	public static native void callIPPIPV(long __functionAddress, int param0, long param1, long param2, int param3, long param4);
	public static native void callIPPIV(long __functionAddress, int param0, long param1, long param2, int param3);
	public static native long callIPPP(long __functionAddress, int param0, long param1, long param2);
	public static native void callIPPPPIV(long __functionAddress, int param0, long param1, long param2, long param3, long param4, int param5);
	public static native void callIPPPV(long __functionAddress, int param0, long param1, long param2, long param3);
	public static native void callIPPV(long __functionAddress, int param0, long param1, long param2);
	public static native boolean callIPPZ(long __functionAddress, int param0, long param1, long param2);
	public static native void callIPPZV(long __functionAddress, int param0, long param1, long param2, boolean param3);
	public static native void callIPV(long __functionAddress, int param0, long param1);
	public static native boolean callIPZ(long __functionAddress, int param0, long param1);
	public static native short callIS(long __functionAddress, int param0);
	public static native void callISSSSV(long __functionAddress, int param0, short param1, short param2, short param3, short param4);
	public static native void callISSSV(long __functionAddress, int param0, short param1, short param2, short param3);
	public static native void callISSV(long __functionAddress, int param0, short param1, short param2);
	public static native void callISV(long __functionAddress, int param0, short param1);
	public static native void callIV(long __functionAddress, int param0);
	public static native boolean callIZ(long __functionAddress, int param0);
	public static native void callIZIIIPV(long __functionAddress, int param0, boolean param1, int param2, int param3, int param4, long param5);
	public static native void callIZIIPV(long __functionAddress, int param0, boolean param1, int param2, int param3, long param4);
	public static native void callIZV(long __functionAddress, int param0, boolean param1);
	public static native void callIZZZZV(long __functionAddress, int param0, boolean param1, boolean param2, boolean param3, boolean param4);
	public static native long callJ(long __functionAddress);
	public static native void callJIV(long __functionAddress, long param0, int param1);
	public static native void callJV(long __functionAddress, long param0);
	public static native boolean callJZ(long __functionAddress, long param0);
	public static native long callP(long __functionAddress);
	public static native int callPFI(long __functionAddress, long param0, float param1);
	public static native int callPI(long __functionAddress, long param0);
	public static native int callPII(long __functionAddress, long param0, int param1);
	public static native int callPIII(long __functionAddress, long param0, int param1, int param2);
	public static native int callPIIIFFIPI(long __functionAddress, long param0, int param1, int param2, int param3, float param4, float param5, int param6, long param7);
	public static native int callPIIII(long __functionAddress, long param0, int param1, int param2, int param3);
	public static native int callPIIIII(long __functionAddress, long param0, int param1, int param2, int param3, int param4);
	public static native int callPIIIIIII(long __functionAddress, long param0, int param1, int param2, int param3, int param4, int param5, int param6);
	public static native void callPIIIIIIIIIIV(long __functionAddress, long param0, int param1, int param2, int param3, int param4, int param5, int param6, int param7, int param8, int param9, int param10);
	public static native int callPIIIIIIPIIIIIIIIII(long __functionAddress, long param0, int param1, int param2, int param3, int param4, int param5, int param6, long param7, int param8, int param9, int param10, int param11, int param12, int param13, int param14, int param15, int param16);
	public static native int callPIIIPI(long __functionAddress, long param0, int param1, int param2, int param3, long param4);
	public static native long callPIIIPP(long __functionAddress, long param0, int param1, int param2, int param3, long param4);
	public static native int callPIIIPPI(long __functionAddress, long param0, int param1, int param2, int param3, long param4, long param5);
	public static native void callPIIIV(long __functionAddress, long param0, int param1, int param2, int param3);
	public static native long callPIIP(long __functionAddress, long param0, int param1, int param2);
	public static native int callPIIPI(long __functionAddress, long param0, int param1, int param2, long param3);
	public static native int callPIIPPPI(long __functionAddress, long param0, int param1, int param2, long param3, long param4, long param5);
	public static native void callPIIPPV(long __functionAddress, long param0, int param1, int param2, long param3, long param4);
	public static native int callPIJI(long __functionAddress, long param0, int param1, long param2);
	public static native int callPIJPPIPPI(long __functionAddress, long param0, int param1, long param2, long param3, long param4, int param5, long param6, long param7);
	public static native void callPIJV(long __functionAddress, long param0, int param1, long param2);
	public static native long callPIP(long __functionAddress, long param0, int param1);
	public static native int callPIPI(long __functionAddress, long param0, int param1, long param2);
	public static native int callPIPIPIPPI(long __functionAddress, long param0, int param1, long param2, int param3, long param4, int param5, long param6, long param7);
	public static native int callPIPIPPI(long __functionAddress, long param0, int param1, long param2, int param3, long param4, long param5);
	public static native int callPIPJIPPI(long __functionAddress, long param0, int param1, long param2, long param3, int param4, long param5, long param6);
	public static native long callPIPP(long __functionAddress, long param0, int param1, long param2);
	public static native int callPIPPI(long __functionAddress, long param0, int param1, long param2, long param3);
	public static native long callPIPPIPPPP(long __functionAddress, long param0, int param1, long param2, long param3, int param4, long param5, long param6, long param7);
	public static native int callPIPPIPPPPI(long __functionAddress, long param0, int param1, long param2, long param3, int param4, long param5, long param6, long param7, long param8);
	public static native int callPIPPJIPPI(long __functionAddress, long param0, int param1, long param2, long param3, long param4, int param5, long param6, long param7);
	public static native long callPIPPP(long __functionAddress, long param0, int param1, long param2, long param3);
	public static native int callPIPPPI(long __functionAddress, long param0, int param1, long param2, long param3, long param4);
	public static native int callPIPPPIPPI(long __functionAddress, long param0, int param1, long param2, long param3, long param4, int param5, long param6, long param7);
	public static native long callPIPPPP(long __functionAddress, long param0, int param1, long param2, long param3, long param4);
	public static native int callPIPPPPI(long __functionAddress, long param0, int param1, long param2, long param3, long param4, long param5);
	public static native long callPIPPPPP(long __functionAddress, long param0, int param1, long param2, long param3, long param4, long param5);
	public static native long callPIPPPPPP(long __functionAddress, long param0, int param1, long param2, long param3, long param4, long param5, long param6);
	public static native void callPIPPV(long __functionAddress, long param0, int param1, long param2, long param3);
	public static native void callPIPV(long __functionAddress, long param0, int param1, long param2);
	public static native boolean callPIPZ(long __functionAddress, long param0, int param1, long param2);
	public static native boolean callPIZ(long __functionAddress, long param0, int param1);
	public static native long callPJIIIPP(long __functionAddress, long param0, long param1, int param2, int param3, int param4, long param5);
	public static native int callPJIIPPI(long __functionAddress, long param0, long param1, int param2, int param3, long param4, long param5);
	public static native long callPJIIPPP(long __functionAddress, long param0, long param1, int param2, int param3, long param4, long param5);
	public static native long callPJIPP(long __functionAddress, long param0, long param1, int param2, long param3);
	public static native int callPJIPPI(long __functionAddress, long param0, long param1, int param2, long param3, long param4);
	public static native long callPJIPPP(long __functionAddress, long param0, long param1, int param2, long param3, long param4);
	public static native long callPJPIP(long __functionAddress, long param0, long param1, long param2, int param3);
	public static native long callPJPPPP(long __functionAddress, long param0, long param1, long param2, long param3, long param4);
	public static native long callPJPPPPP(long __functionAddress, long param0, long param1, long param2, long param3, long param4, long param5);
	public static native long callPJPPPPPPP(long __functionAddress, long param0, long param1, long param2, long param3, long param4, long param5, long param6, long param7);
	public static native long callPJPPPPPPPPP(long __functionAddress, long param0, long param1, long param2, long param3, long param4, long param5, long param6, long param7, long param8, long param9);
	public static native long callPP(long __functionAddress, long param0);
	public static native int callPPFI(long __functionAddress, long param0, long param1, float param2);
	public static native int callPPI(long __functionAddress, long param0, long param1);
	public static native int callPPII(long __functionAddress, long param0, long param1, int param2);
	public static native int callPPIIII(long __functionAddress, long param0, long param1, int param2, int param3, int param4);
	public static native void callPPIIIIIIPIIIIIIIIIV(long __functionAddress, long param0, long param1, int param2, int param3, int param4, int param5, int param6, int param7, long param8, int param9, int param10, int param11, int param12, int param13, int param14, int param15, int param16, int param17);
	public static native boolean callPPIIIIPZ(long __functionAddress, long param0, long param1, int param2, int param3, int param4, int param5, long param6);
	public static native boolean callPPIIIIZ(long __functionAddress, long param0, long param1, int param2, int param3, int param4, int param5);
	public static native long callPPIIIP(long __functionAddress, long param0, long param1, int param2, int param3, int param4);
	public static native boolean callPPIIIPZ(long __functionAddress, long param0, long param1, int param2, int param3, int param4, long param5);
	public static native long callPPIIPP(long __functionAddress, long param0, long param1, int param2, int param3, long param4);
	public static native int callPPIIPPI(long __functionAddress, long param0, long param1, int param2, int param3, long param4, long param5);
	public static native boolean callPPIIZ(long __functionAddress, long param0, long param1, int param2, int param3);
	public static native int callPPIJI(long __functionAddress, long param0, long param1, int param2, long param3);
	public static native int callPPIJIPPI(long __functionAddress, long param0, long param1, int param2, long param3, int param4, long param5, long param6);
	public static native long callPPIJPPIPPPP(long __functionAddress, long param0, long param1, int param2, long param3, long param4, long param5, int param6, long param7, long param8, long param9);
	public static native long callPPIJPPPPIPPPP(long __functionAddress, long param0, long param1, int param2, long param3, long param4, long param5, long param6, long param7, int param8, long param9, long param10, long param11);
	public static native long callPPIP(long __functionAddress, long param0, long param1, int param2);
	public static native int callPPIPI(long __functionAddress, long param0, long param1, int param2, long param3);
	public static native long callPPIPIP(long __functionAddress, long param0, long param1, int param2, long param3, int param4);
	public static native long callPPIPP(long __functionAddress, long param0, long param1, int param2, long param3);
	public static native int callPPIPPI(long __functionAddress, long param0, long param1, int param2, long param3, long param4);
	public static native long callPPIPPP(long __functionAddress, long param0, long param1, int param2, long param3, long param4);
	public static native int callPPIPPPI(long __functionAddress, long param0, long param1, int param2, long param3, long param4, long param5);
	public static native int callPPIPPPIPPI(long __functionAddress, long param0, long param1, int param2, long param3, long param4, long param5, int param6, long param7, long param8);
	public static native int callPPIPPPPPI(long __functionAddress, long param0, long param1, int param2, long param3, long param4, long param5, long param6, long param7);
	public static native int callPPIPPPPPIPPI(long __functionAddress, long param0, long param1, int param2, long param3, long param4, long param5, long param6, long param7, int param8, long param9, long param10);
	public static native int callPPIPPPPPPPPIPPI(long __functionAddress, long param0, long param1, int param2, long param3, long param4, long param5, long param6, long param7, long param8, long param9, long param10, int param11, long param12, long param13);
	public static native void callPPIPV(long __functionAddress, long param0, long param1, int param2, long param3);
	public static native boolean callPPIPZ(long __functionAddress, long param0, long param1, int param2, long param3);
	public static native void callPPIV(long __functionAddress, long param0, long param1, int param2);
	public static native boolean callPPIZ(long __functionAddress, long param0, long param1, int param2);
	public static native long callPPJPP(long __functionAddress, long param0, long param1, long param2, long param3);
	public static native long callPPP(long __functionAddress, long param0, long param1);
	public static native int callPPPI(long __functionAddress, long param0, long param1, long param2);
	public static native void callPPPIIPPPV(long __functionAddress, long param0, long param1, long param2, int param3, int param4, long param5, long param6, long param7);
	public static native long callPPPIP(long __functionAddress, long param0, long param1, long param2, int param3);
	public static native long callPPPIPP(long __functionAddress, long param0, long param1, long param2, int param3, long param4);
	public static native int callPPPIPPI(long __functionAddress, long param0, long param1, long param2, int param3, long param4, long param5);
	public static native boolean callPPPIPZ(long __functionAddress, long param0, long param1, long param2, int param3, long param4);
	public static native boolean callPPPIZ(long __functionAddress, long param0, long param1, long param2, int param3);
	public static native long callPPPJPPP(long __functionAddress, long param0, long param1, long param2, long param3, long param4, long param5);
	public static native long callPPPP(long __functionAddress, long param0, long param1, long param2);
	public static native int callPPPPI(long __functionAddress, long param0, long param1, long param2, long param3);
	public static native int callPPPPIPPIPPI(long __functionAddress, long param0, long param1, long param2, long param3, int param4, long param5, long param6, int param7, long param8, long param9);
	public static native void callPPPPIV(long __functionAddress, long param0, long param1, long param2, long param3, int param4);
	public static native long callPPPPP(long __functionAddress, long param0, long param1, long param2, long param3);
	public static native int callPPPPPI(long __functionAddress, long param0, long param1, long param2, long param3, long param4);
	public static native int callPPPPPIPPI(long __functionAddress, long param0, long param1, long param2, long param3, long param4, int param5, long param6, long param7);
	public static native int callPPPPPPI(long __functionAddress, long param0, long param1, long param2, long param3, long param4, long param5);
	public static native int callPPPPPPIPPI(long __functionAddress, long param0, long param1, long param2, long param3, long param4, long param5, int param6, long param7, long param8);
	public static native int callPPPPPPPPPPIPPI(long __functionAddress, long param0, long param1, long param2, long param3, long param4, long param5, long param6, long param7, long param8, long param9, int param10, long param11, long param12);
	public static native boolean callPPPPPZ(long __functionAddress, long param0, long param1, long param2, long param3, long param4);
	public static native void callPPPPV(long __functionAddress, long param0, long param1, long param2, long param3);
	public static native boolean callPPPPZ(long __functionAddress, long param0, long param1, long param2, long param3);
	public static native void callPPPV(long __functionAddress, long param0, long param1, long param2);
	public static native boolean callPPPZ(long __functionAddress, long param0, long param1, long param2);
	public static native void callPPV(long __functionAddress, long param0, long param1);
	public static native boolean callPPZ(long __functionAddress, long param0, long param1);
	public static native void callPV(long __functionAddress, long param0);
	public static native boolean callPZ(long __functionAddress, long param0);
	public static native void callSSSSV(long __functionAddress, short param0, short param1, short param2, short param3);
	public static native void callSSSV(long __functionAddress, short param0, short param1, short param2);
	public static native void callSSV(long __functionAddress, short param0, short param1);
	public static native void callSV(long __functionAddress, short param0);
	public static native void callV(long __functionAddress);
	public static native boolean callZ(long __functionAddress);
	public static native void callZV(long __functionAddress, boolean param0);
	public static native void callZZZZV(long __functionAddress, boolean param0, boolean param1, boolean param2, boolean param3);
	public static native double invokeD(long __functionAddress);
	public static native void invokeDV(long __functionAddress, double param0);
	public static native void invokeFV(long __functionAddress, float param0);
	public static native int invokeI(long __functionAddress);
	public static native double invokeID(long __functionAddress, int param0);
	public static native float invokeIF(long __functionAddress, int param0);
	public static native void invokeIFFFV(long __functionAddress, int param0, float param1, float param2, float param3);
	public static native void invokeIFV(long __functionAddress, int param0, float param1);
	public static native int invokeII(long __functionAddress, int param0);
	public static native void invokeIIDDDV(long __functionAddress, int param0, int param1, double param2, double param3, double param4);
	public static native void invokeIIDV(long __functionAddress, int param0, int param1, double param2);
	public static native void invokeIIFFFV(long __functionAddress, int param0, int param1, float param2, float param3, float param4);
	public static native void invokeIIFV(long __functionAddress, int param0, int param1, float param2);
	public static native void invokeIIIIIIPV(long __functionAddress, int param0, int param1, int param2, int param3, int param4, int param5, long param6);
	public static native void invokeIIIIIPV(long __functionAddress, int param0, int param1, int param2, int param3, int param4, long param5);
	public static native void invokeIIIIIV(long __functionAddress, int param0, int param1, int param2, int param3, int param4);
	public static native void invokeIIIV(long __functionAddress, int param0, int param1, int param2);
	public static native void invokeIIJJJV(long __functionAddress, int param0, int param1, long param2, long param3, long param4);
	public static native void invokeIIJV(long __functionAddress, int param0, int param1, long param2);
	public static native long invokeIIP(long __functionAddress, int param0, int param1);
	public static native void invokeIIPIIV(long __functionAddress, int param0, int param1, long param2, int param3, int param4);
	public static native long invokeIIPPPP(long __functionAddress, int param0, int param1, long param2, long param3, long param4);
	public static native void invokeIIPPPV(long __functionAddress, int param0, int param1, long param2, long param3, long param4);
	public static native void invokeIIPV(long __functionAddress, int param0, int param1, long param2);
	public static native void invokeIIV(long __functionAddress, int param0, int param1);
	public static native long invokeIP(long __functionAddress, int param0);
	public static native long invokeIPP(long __functionAddress, int param0, long param1);
	public static native void invokeIPPPV(long __functionAddress, int param0, long param1, long param2, long param3);
	public static native void invokeIPV(long __functionAddress, int param0, long param1);
	public static native void invokeIV(long __functionAddress, int param0);
	public static native boolean invokeIZ(long __functionAddress, int param0);
	public static native long invokeP(long __functionAddress);
	public static native byte invokePB(long __functionAddress, long param0);
	public static native long invokePBBPP(long __functionAddress, long param0, byte param1, byte param2, long param3);
	public static native void invokePDDV(long __functionAddress, long param0, double param1, double param2);
	public static native void invokePFV(long __functionAddress, long param0, float param1);
	public static native int invokePI(long __functionAddress, long param0);
	public static native int invokePII(long __functionAddress, long param0, int param1);
	public static native void invokePIIIIV(long __functionAddress, long param0, int param1, int param2, int param3, int param4);
	public static native long invokePIIIP(long __functionAddress, long param0, int param1, int param2, int param3);
	public static native boolean invokePIIIZ(long __functionAddress, long param0, int param1, int param2, int param3);
	public static native long invokePIIP(long __functionAddress, long param0, int param1, int param2);
	public static native void invokePIIPV(long __functionAddress, long param0, int param1, int param2, long param3);
	public static native void invokePIIV(long __functionAddress, long param0, int param1, int param2);
	public static native long invokePIP(long __functionAddress, long param0, int param1);
	public static native void invokePIPPV(long __functionAddress, long param0, int param1, long param2, long param3);
	public static native void invokePIV(long __functionAddress, long param0, int param1);
	public static native long invokePP(long __functionAddress, long param0);
	public static native byte invokePPB(long __functionAddress, long param0, long param1);
	public static native long invokePPBBP(long __functionAddress, long param0, long param1, byte param2, byte param3);
	public static native int invokePPI(long __functionAddress, long param0, long param1);
	public static native long invokePPIP(long __functionAddress, long param0, long param1, int param2);
	public static native void invokePPIV(long __functionAddress, long param0, long param1, int param2);
	public static native long invokePPP(long __functionAddress, long param0, long param1);
	public static native void invokePPPBBV(long __functionAddress, long param0, long param1, long param2, byte param3, byte param4);
	public static native byte invokePPPBPB(long __functionAddress, long param0, long param1, long param2, byte param3, long param4);
	public static native int invokePPPI(long __functionAddress, long param0, long param1, long param2);
	public static native byte invokePPPIB(long __functionAddress, long param0, long param1, long param2, int param3);
	public static native void invokePPPIBBV(long __functionAddress, long param0, long param1, long param2, int param3, byte param4, byte param5);
	public static native long invokePPPIP(long __functionAddress, long param0, long param1, long param2, int param3);
	public static native void invokePPPIV(long __functionAddress, long param0, long param1, long param2, int param3);
	public static native long invokePPPP(long __functionAddress, long param0, long param1, long param2);
	public static native byte invokePPPPB(long __functionAddress, long param0, long param1, long param2, long param3);
	public static native long invokePPPPP(long __functionAddress, long param0, long param1, long param2, long param3);
	public static native int invokePPPPPI(long __functionAddress, long param0, long param1, long param2, long param3, long param4);
	public static native int invokePPPPPPI(long __functionAddress, long param0, long param1, long param2, long param3, long param4, long param5);
	public static native void invokePPPPPV(long __functionAddress, long param0, long param1, long param2, long param3, long param4);
	public static native void invokePPPPV(long __functionAddress, long param0, long param1, long param2, long param3);
	public static native void invokePPPV(long __functionAddress, long param0, long param1, long param2);
	public static native void invokePPV(long __functionAddress, long param0, long param1);
	public static native boolean invokePPZ(long __functionAddress, long param0, long param1);
	public static native void invokePV(long __functionAddress, long param0);
	public static native boolean invokePZ(long __functionAddress, long param0);
	public static native void invokeV(long __functionAddress);

}
